<?php

namespace AltSolution\Admin\EmailTemplate;

use Twig\Environment;
use Twig\Loader\ArrayLoader;
use Twig\Extension\AbstractExtension;

class TwigRender implements RenderInterface
{
    public function render($tpl, array $viewData = null)
    {
        $twig_env = $this->bootTwig([
            'body' => $tpl,
        ]);
        return $twig_env->render('body', $viewData ?: []);
    }

    /**
     * @param array $templates
     * @return Environment
     * @throws \Exception
     */
    protected function bootTwig(array $templates)
    {
        $twig_env = new Environment(new ArrayLoader($templates));

        $extensions = config('twigbridge.extensions.enabled');
        foreach ($extensions as $extension) {
            if (is_string($extension)) {
                try {
                    $extension = app($extension);
                } catch (\Exception $e) {
                    throw new \Exception("Cannot instantiate Twig extension '$extension': " . $e->getMessage());
                }
            } elseif (is_callable($extension)) {
                $extension = $extension(app(), $twig_env);
            } elseif (!is_a($extension, AbstractExtension::class)) {
                throw new \InvalidArgumentException('Incorrect extension type');
            }
            $twig_env->addExtension($extension);
        }

        return $twig_env;
    }
}
