<?php

namespace AltSolution\Admin\Form;

use Illuminate\Support\Str;

abstract class AbstractComponent implements ComponentInterface
{
    protected $enumerable = true;
    protected $visible = true;
    // TODO:
    // protected $compound = false;
    // protected $mapped = false;
    protected $viewNs = 'admin::form.component';
    protected $view = 'empty';

    /**
     * @var string
     */
    protected $name;
    /**
     * @var array
     */
    protected $options;
    protected $viewOptions;
    /**
     * @var Form
     */
    private $form;

    /**
     * @param string $name
     * @param array $options
     */
    public function __construct($name, array $options = [])
    {
        $this->name = $name;
        $this->options = $options;
        $this->viewOptions = [];
    }

    public function setForm(Form $form)
    {
        $this->form = $form;
    }

    public function getForm()
    {
        return $this->form;
    }

    public function getName()
    {
        return $this->name;
    }

    /**
     * Get initial option
     * @param string $name
     * @param mixed $default
     * @return mixed
     */
    public function getOption($name, $default = null)
    {
        if (isset($this->viewOptions[$name])) {
            return $this->viewOptions[$name];
        }
        if (isset($this->options[$name])) {
            return $this->options[$name];
        }
        return value($default);
    }

    public function setOption($name, $value)
    {
        $this->options[$name] = $value;
    }

    /**
     * Build data for view
     * @param array $options
     * @return array
     */
    protected function viewData(array $options)
    {
        $this->viewOptions = $options;

        $form = $this->form;
        $field = $this;

        $id = $this->getOption('id', 'id_' . Str::random());
        $name = $this->transformName($this->name);
        // TODO: deprecate, god access
        $options = array_merge($this->options, $this->viewOptions);

        // default label
        if (empty($options['label'])) {
            $options['label'] = $this->name;
        }

        $baseData = compact('form', 'field', 'id', 'name', 'options');
        $customData = $this->customViewData();

        return $baseData + $customData;
    }

    protected function customViewData()
    {
        return [];
    }

    /**
     * Generate name for html. Example: foo.bar.baz ==> foo[bar][baz]
     * @param $name
     * @return string
     */
    protected function transformName($name)
    {
        $nameParts = explode('.', $name);
        $nameTmp = [];
        $first = true;
        foreach ($nameParts as $namePart) {
            if ($first) {
                $first = false;
                $nameTmp[] = $namePart;
            } else {
                $nameTmp[] = '[' . $namePart . ']';
            }
        }
        return implode('', $nameTmp);
    }

    public function render(array $options = [])
    {
        $viewData = $this->viewData($options);
        $viewFile = $this->viewNs . '.' . $this->view;

        return view($viewFile, $viewData)->render();
    }

    public function isEnumerable()
    {
        return $this->enumerable;
    }
    
    public function isVisible()
    {
        return $this->visible;
    }
}