<?php

namespace AltSolution\Admin\Http\Controllers;

use AltSolution\Admin\Backup;
use AltSolution\Admin\Forms\BackupForm;
use Illuminate\Http\Request;

class BackupController extends Controller
{
    public function index()
    {
        $this->authorize('permission', 'backup');

        $form = cms_create_form(BackupForm::class);

        $this->layout
            ->setMenuSection('debug')
            ->setTitle(trans('admin::backup.title'));

        return view('admin::backup.index', compact('form'));
    }

    public function postIndex(Request $request)
    {
        $this->authorize('permission', 'backup');

        $form = cms_create_form(BackupForm::class);
        $form->validate($request);

        $type = $request->type;

        try {
            if ($type == 'database') {
                $backuper = app(Backup\Database::class);
                $path = $backuper->createDump();
                return response()->download($path)->deleteFileAfterSend(true);
            } elseif ($type == 'code') {
                //throw new \Exception('Not implemented');
                $archiver = app(Backup\Code::class);
                $path = $archiver->createArchive();
                return response()->download($path)->deleteFileAfterSend(true);
            }
        } catch (\Exception $e) {
            $this->layout->addNotify('error', $e->getMessage());
        }

        return redirect()->route('admin::backup')->withInput();
    }

}
