<?php

namespace AltSolution\Admin\Http\Controllers;

use AltSolution\Admin\Services\DashboardService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Artisan;

class IndexController extends Controller
{
    public function index()
    {
        $startPage = cms_option('admin_start_page');
        if ($startPage) {
            return redirect($startPage);
        }

        return redirect()->route('admin::dashboard');
    }

    public function dashboard(DashboardService $dashboardService)
    {
        $hasFullAccess = \Gate::allows('permission', 'debug');

        $info = [
            'general' => $dashboardService->getGeneralInfo(),
            'assets' => $dashboardService->getAssetsInfo(),
            'production' => $dashboardService->getProductionInfo(),
            'mail' => $dashboardService->getMailInfo(),
            'seo' => $dashboardService->getSeoInfo(),
        ];

        /*
        $console = app(\App\Console\Kernel::class);
        $schedule = app(\Illuminate\Console\Scheduling\Schedule::class);
        $events = collect($schedule->events())->map(function(\Illuminate\Console\Scheduling\Event $event) {
            //$commandParts = explode(' ', $event->command, 3);

            return (object)[
                'expression' => $event->expression,
                //'command' => $commandParts[2],
                'command' => $event->command,
                'next_run_at' => $event->nextRunDate(),
            ];
        });
        // \Illuminate\Console\Events\CommandStarting
        // \Illuminate\Console\Events\CommandFinished
        */


        $this->layout
            ->setMenuSection('dashboard')
            ->hideBreadcrumbs()
            ->setTitle(trans('admin::dashboard.title'));

        return view('admin::dashboard.index', compact('info'), [
            'full_access' => $hasFullAccess,
        ]);
    }

    public function action(Request $request, DashboardService $dashboardService)
    {
        $this->authorize('permission', 'debug');

        $action = $request->input('action');

        $success = true;
        $output = null;

        if ($action == 'fix-config') {
            $code = Artisan::call('config:cache', []);
            $success = $code == 0;
            $output = Artisan::output();
        } elseif ($action == 'fix-routes') {
            $code = Artisan::call('route:cache', []);
            $success = $code == 0;
            $output = Artisan::output();
        } elseif ($action == 'fix-assets') {
            $link = public_path('vendor/admin');
            if (!file_exists($link)) {
                $target = base_path('vendor/altsolution/cms-core/assets');
                app('files')->link($target, $link);
                //app('files')->relativeLink($target, $link);
            }
        }

        return compact('success', 'output');
    }
}
