<?php

namespace AltSolution\Admin\Http\Controllers;

use AltSolution\Admin\Modules\Menu\MenuItemFormInterface;
use AltSolution\Admin\Modules\Menu\MenuItemModelInterface;
use AltSolution\Admin\Modules\Menu\MenuModelInterface;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;

class MenuItemController extends Controller
{
    public function getIndex($menuId)
    {
        $this->authorize('permission', 'menuitem.list');

        $menu = app(MenuModelInterface::class)->query()->find($menuId);
        if (!$menu) {
            return redirect()->route('admin::menu_list');
        }
        $items = app(MenuItemModelInterface::class)
            ->where('menu_id', $menu->id)
            ->where('parent_id', 0)
            ->orderBy('sort')
            ->get();

        $this->layout
            ->setMenuSection('menu')
            ->setTitle(trans('admin::menu.items', ['menu' => $menu->description]))
            ->addBreadcrumb(trans('admin::menu.title'), route('admin::menu_list'));
        return view('admin::menu_item.list', ['items' => $items, 'parentMenu' => $menu]);
    }

    public function getEdit($menuId, $id = null)
    {
        $this->authorize('permission', 'menuitem.edit');

        $menu = app(MenuModelInterface::class)->query()->find($menuId);
        if (!$menu) {
            return redirect()->route('admin::menu_list');
        }

        $item = null;
        if ($id) {
            $item = app(MenuItemModelInterface::class)->query()->where('menu_id', $menu->id)->find($id);
        }
        $parent_items = app(MenuItemModelInterface::class)
            ->where('menu_id', $menu->id)
            ->parent($id)
            ->orderBy('sort')
            ->get();
        $form = cms_create_form(MenuItemFormInterface::class, $item, [
            'parent_items' => $parent_items,
        ]);
        $entities = cms_system()->getEntity()->getItems();

        $data = [
            'parentMenu' => $menu,
            'item' => $item,
            'entities' => $entities,
            //
            'form' => $form,
        ];

        $this->layout
            ->setMenuSection('menu')
            ->setTitle(trans($id ? 'admin::menu.item_edit' : 'admin::menu.item_add'))
            ->addBreadcrumb(trans('admin::menu.title'), route('admin::menu_list'))
            ->addBreadcrumb(trans('admin::menu.items', ['menu' => $menu->description]), route('admin::menu_item_list', ['id' => $menu->id]));
        return view('admin::menu_item.edit', $data);
    }

    public function postSave(Request $request)
    {
        $this->authorize('permission', 'menuitem.edit');

        $itemId = $request->input('id');
        $item = app(MenuItemModelInterface::class)->query()->firstOrNew(['id' => $request->id]);
        $form = cms_create_form(MenuItemFormInterface::class, $item);
        $validData = $form->validate($request);

        $item->fill($validData);
        $item->target = (bool)$request->target;
        if (!$item->exists) {
            $item->sort = app(MenuItemModelInterface::class)->query()->max('sort') + 1;
        }
        event('admin.model.menu-item.saving', $item);
        $item->save();
        event('admin.model.menu-item.saved', $item);

        $this->layout->addNotify('success', trans('admin::menu.item_saved'));

        if ($request->button_apply) {
            return redirect()->route('admin::menu_item_edit', ['id' => $request->menu_id, 'id2' => $item->id]);
        }

        return redirect()->route('admin::menu_item_list', $request->menu_id);
    }

    public function action(Request $request)
    {
        $action = $request->input('action');
        $ids = $request->input('ids');
        if (!$ids) {
            return;
        }

        if ($action == 'delete') {
            $this->authorize('permission', 'menuitem.delete');

            foreach ($ids as $id) {
                $item = app(MenuItemModelInterface::class)->query()->find($id);
                if (!$item) {
                    continue;
                }

                event('admin.model.menu-item.deleting', $item);
                $item->delete();
                event('admin.model.menu-item.deleted', $item);
            }
        } elseif ($action == 'move') {
            $this->authorize('permission', 'menuitem.edit');

            $order = 0;
            foreach ($ids as $node) {
                $itemId = Arr::get($node, 'id');
                $item = app(MenuItemModelInterface::class)->query()->find($itemId);
                if ($item) {
                    // TODO: make nullable
                    $parentId = Arr::get($node, 'parent_id', 0);
                    $item->parent_id = $parentId;
                    $item->sort = $order++;

                    event('admin.model.menu-item.saving', $item);
                    $item->save();
                    event('admin.model.menu-item.saved', $item);
                }
            }

            $this->layout->addNotify('success', trans('admin::common.save_sort'));
        }
    }
}
