<?php

namespace AltSolution\Admin\Twig\Extension;

use AltSolution\Admin\Seo\SeoInterface;
use AltSolution\Admin\Seo\SeoManagerInterface;
use Twig\Extension\AbstractExtension;
use Twig\TwigFilter;
use Twig\TwigFunction;

class SeoExtension extends AbstractExtension
{
    /**
     * @var SeoManagerInterface
     */
    private $seoManager;

    /**
     * Get Seo manager instance. Avoid using of constructor due lazy load
     * @return SeoManagerInterface
     */
    private function getSeoManager()
    {
        if ($this->seoManager === null) {
            $this->seoManager = app(SeoManagerInterface::class);
        }
        return $this->seoManager;
    }

    public function getName()
    {
        return 'AltSolution_CMS_SEO';
    }

    public function getFunctions()
    {
        return [
            new TwigFunction('seo_title', [$this, 'seoTitle'], ['is_safe' => ['html']]),
            new TwigFunction('seo_meta_name', [$this, 'seoMetaName'], ['is_safe' => ['html']]),
            new TwigFunction('seo_meta_property', [$this, 'seoMetaProperty'], ['is_safe' => ['html']]),
            new TwigFunction('seo_custom', [$this, 'seoCustom'], ['is_safe' => ['html']]),
            new TwigFunction('seo_render', [$this, 'seoRender'], ['is_safe' => ['html']]),
            new TwigFunction('seo_admin_fieldset', [$this, 'seoAdminFieldset'], ['is_safe' => ['html']]),
        ];
    }

    public function seoTitle()
    {
        return $this->getSeoManager()->getTitle();
    }

    public function seoMetaName($name)
    {
        return $this->getSeoManager()->getMetaName($name);
    }

    public function seoMetaProperty($property)
    {
        return $this->getSeoManager()->getMetaProperty($property);
    }

    public function seoCustom($key)
    {
        return $this->getSeoManager()->getCustom($key);
    }

    public function seoRender()
    {
        return $this->getSeoManager()->renderAll();
    }

    public function seoAdminFieldset(SeoInterface $model, $locale)
    {
        return $this->getSeoManager()->renderFieldset($model, $locale);
    }

    public function getFilters()
    {
        return [
            new TwigFilter('seo_noindex', [$this, 'seoNoIndex'], ['is_safe' => ['html']]),
        ];
    }

    public function seoNoIndex($value)
    {
        // Yandex bot ignore this text.
        $str = '<noindex>' . $value . '</noindex>';

        return $str;
    }
}
